@@ -1,14 +1,15 @@
 __all__ = ['thread',
            'thread_pool',
            'process',
-           #'process_pool',
            'synchronized',
+           'Task',
            'ThreadPool',
            'PebbleError',
            'ProcessPool',
            'TimeoutError',
            'TaskCancelled']
 
-from .pebble import PebbleError, TimeoutError, synchronized
-from .process import ProcessPool, process, TaskCancelled
+from .pebble import PebbleError, TaskCancelled, TimeoutError
+from .pebble import Task, synchronized
+from .process import ProcessPool, process
 from .thread import thread, thread_pool, ThreadPool
