@@ -14,12 +14,11 @@
 # along with Pebble.  If not, see <http://www.gnu.org/licenses/>.
 
 
-from time import sleep
-from uuid import uuid4
+from time import time, sleep
 from inspect import isclass
-from traceback import format_exc
+from traceback import format_exc, print_exc
 from itertools import count
-from threading import Thread, Event
+from threading import Thread
 from collections import Callable
 from functools import update_wrapper
 try:  # Python 2
@@ -27,7 +26,7 @@ try:  # Python 2
 except:  # Python 3
     from queue import Queue
 
-from .pebble import TimeoutError, TaskCancelled
+from .pebble import Task, TimeoutError
 
 
 STOPPED = 0
@@ -109,78 +108,32 @@ class Worker(Thread):
                 error.traceback = format_exc()
 
         while self.limit == 0 or next(counter) < self.limit:
-            function, task, args, kwargs = self.queue.get()
-            if function is None:
+            task = self.queue.get()
+            if task is None:  # worker terminated
                 self.queue.task_done()
                 return
+            function = task._function
+            args = task._args
+            kwargs = task._kwargs
             try:
-                results = function(*args, **kwargs)
+                if not task._cancelled:
+                    task._timestamp = time()
+                    results = function(*args, **kwargs)
             except Exception as err:
                 error = err
                 error.traceback = format_exc()
             finally:
-                self.queue.task_done()
                 task._set(error is not None and error or results)
+                if task._callback is not None:
+                    try:
+                        task._callback(task)
+                    except:
+                        print_exc()
+                self.queue.task_done()
                 error = None
                 results = None
 
 
-class Task(object):
-    """Handler to the ongoing task."""
-    def __init__(self, task_nr, callback):
-        self.id = uuid4()
-        self.number = task_nr
-        self._ready = False
-        self._cancelled = False
-        self._results = None
-        self._event = Event()
-        self._callback = callback
-
-    def __str__(self):
-        return self.__repr__()
-
-    def __repr__(self):
-        return "%s (Task-%d, %s)" % (self.__class__, self.number, self.id)
-
-    @property
-    def ready(self):
-        return self._ready
-
-    @property
-    def cancelled(self):
-        return self._cancelled
-
-    def get(self, timeout=None):
-        """Retrieves the produced results.
-
-        If the executed code raised an error it will be re-raised.
-
-        """
-        self._event.wait(timeout)
-        if self._ready:
-            if (isinstance(self._results, BaseException)):
-                raise self._results
-            else:
-                return self._results
-        else:
-            raise TimeoutError("Task is still running")
-
-    def cancel(self):
-        """Cancels the Task dropping the results."""
-        if not self._ready:
-            self._cancelled = True
-            self._set(TaskCancelled("Task has been cancelled"))
-        else:
-            raise RuntimeError('A completed task cannot be cancelled')
-
-    def _set(self, results):
-        self._results = results
-        self._ready = True
-        self._event.set()
-        if self._callback is not None and not self._cancelled:
-            self._callback(self)
-
-
 class ThreadPool(object):
     def __init__(self, workers=1, task_limit=0, queue=None, queueargs=None,
                  initializer=None, initargs=None):
@@ -229,7 +182,7 @@ class ThreadPool(object):
         for w in self._pool:
             w.limit = - 1
         for w in self._pool:
-            self._queue.put((None, None, None, None))
+            self._queue.put(None)
 
     def close(self):
         """Close the pool allowing all queued tasks to be performed."""
@@ -239,7 +192,7 @@ class ThreadPool(object):
         for w in self._pool:
             w.limit = - 1
         for w in self._pool:
-            self._queue.put((None, None, None, None))
+            self._queue.put(None)
 
     def join(self, timeout=0):
         """Joins the pool waiting until all workers exited.
@@ -284,8 +237,8 @@ class ThreadPool(object):
             raise RuntimeError('The Pool is not running')
         if not isinstance(function, Callable):
             raise ValueError('function must be callable')
-        task = Task(next(self._counter), callback)
-        self._queue.put((function, task, args, kwargs))
+        task = Task(next(self._counter), function, args, kwargs, callback, 0)
+        self._queue.put(task)
 
         return task
 
@@ -299,8 +252,9 @@ class ThreadWrapper(object):
         update_wrapper(self, function)
 
     def __call__(self, *args, **kwargs):
-        t = Task(next(self._counter), self.callback)
-        q = DummyQueue((self._function, t, args, kwargs))
+        t = Task(next(self._counter),
+                 self._function, args, kwargs, self.callback, 0)
+        q = DummyQueue(t)
         w = Worker(q, 1, None, None)
         w.start()
         return t
